using System;
using System.Collections.Generic;

using Windows.Storage;

using Microsoft.UI.Xaml.Controls;
using Microsoft.UI.Xaml;

namespace TestProject;

/// <summary>
///  
/// </summary>
public sealed partial class MainPage : Page
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - MainPage()

    /// <summary>
    /// 
    /// </summary>
    public MainPage()
    {
        InitializeComponent();

        this.refreshButton.Click  += refreshButton_Click;
        this.treeView.Expanding   += treeView_Expanding;
        this.treeView.Collapsed   += treeView_Collapsed;
        this.treeView.ItemInvoked += treeView_ItemInvoked;

        InitializeTreeView();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region Refresh ư Ŭ óϱ - refreshButton_Click(sender, e)

    /// <summary>
    /// Refresh ư Ŭ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void refreshButton_Click(object sender, RoutedEventArgs e)
    {
        this.treeView.RootNodes.Clear();

        InitializeTreeView();
    }

    #endregion
    #region Ʈ Ȯ óϱ - treeView_Expanding(sender, e)

    /// <summary>
    /// Ʈ Ȯ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void treeView_Expanding(TreeView sender, TreeViewExpandingEventArgs e)
    {
        if(e.Node.HasUnrealizedChildren)
        {
            AddChildTreeViewNode(e.Node);
        }
    }

    #endregion
    #region Ʈ ҽ óϱ - treeView_Collapsed(sender, e)

    /// <summary>
    /// Ʈ ҽ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void treeView_Collapsed(TreeView sender, TreeViewCollapsedEventArgs e)
    {
        e.Node.Children.Clear();

        e.Node.HasUnrealizedChildren = true;
    }

    #endregion
    #region Ʈ ׸ ȣ óϱ - treeView_ItemInvoked(sender, e)

    /// <summary>
    /// Ʈ ׸ ȣ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void treeView_ItemInvoked(TreeView sender, TreeViewItemInvokedEventArgs e)
    {
        TreeViewNode treeViewNode = e.InvokedItem as TreeViewNode;

        if(treeViewNode.Content is IStorageItem storageItem)
        {
            this.fileNameTextBlock.Text  = storageItem.Name;
            this.filePathTextBlock.Text  = storageItem.Path;
            this.treeDepthTextBlock.Text = treeViewNode.Depth.ToString();

            if(treeViewNode.Content is StorageFolder)
            {
                treeViewNode.IsExpanded = !treeViewNode.IsExpanded;
            }
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region Ʈ ʱȭϱ - InitializeTreeView()

    /// <summary>
    /// Ʈ ʱȭϱ
    /// </summary>
    private void InitializeTreeView()
    {
        StorageFolder pictureStorageFolder = KnownFolders.PicturesLibrary;

        TreeViewNode pictureTreeViewNode = new TreeViewNode();

        pictureTreeViewNode.Content               = pictureStorageFolder;
        pictureTreeViewNode.IsExpanded            = true;
        pictureTreeViewNode.HasUnrealizedChildren = true;

        this.treeView.RootNodes.Add(pictureTreeViewNode);

        AddChildTreeViewNode(pictureTreeViewNode);

        StorageFolder musicStorageFolder = KnownFolders.MusicLibrary;

        TreeViewNode musicTreeViewNode = new TreeViewNode();

        musicTreeViewNode.Content               = musicStorageFolder;
        musicTreeViewNode.IsExpanded            = true;
        musicTreeViewNode.HasUnrealizedChildren = true;

        this.treeView.RootNodes.Add(musicTreeViewNode);

        AddChildTreeViewNode(musicTreeViewNode);
    }

    #endregion
    #region ڽ Ʈ  ߰ϱ - AddChildTreeViewNode(parentTreeViewNode)

    /// <summary>
    /// ڽ Ʈ  ߰ϱ
    /// </summary>
    /// <param name="parentTreeViewNode">θ Ʈ </param>
    private async void AddChildTreeViewNode(TreeViewNode parentTreeViewNode)
    {
        StorageFolder storageFolder = null;

        if(parentTreeViewNode.Content is StorageFolder && parentTreeViewNode.HasUnrealizedChildren == true)
        {
            storageFolder = parentTreeViewNode.Content as StorageFolder;
        }
        else
        {
            return;
        }

        IReadOnlyList<IStorageItem> storageItemList = await storageFolder.GetItemsAsync();

        if(storageItemList.Count == 0)
        {
            return;
        }

        foreach(IStorageItem storageItem in storageItemList)
        {
            TreeViewNode childTreeViewNode = new TreeViewNode();

            childTreeViewNode.Content = storageItem;

            if(storageItem is StorageFolder)
            {
                childTreeViewNode.HasUnrealizedChildren = true;
            }

            parentTreeViewNode.Children.Add(childTreeViewNode);
        }

        parentTreeViewNode.HasUnrealizedChildren = false;
    }

    #endregion
}